@@ -66,6 +66,8 @@ namespace mamba
     std::string unc_url(const std::string& url);
     std::string encode_url(const std::string& url);
     std::string decode_url(const std::string& url);
+    // Only returns a cache name without extension
+    std::string cache_name_from_url(const std::string& url);
 
     class URLHandler
     {
